use anyhow::Result;
use log::Log;

use crate::models::config_model::ConfigModel;
use crate::models::work_model::WorkModel;
use crate::services::{config_service::ConfigService, excel_service::ExcelService, serial_service::SerialService, sqlite_service::SqliteService};
use crate::services::log_service::{FltkLogger, init_logger, LOGGER};

pub struct WorkService {
    pub(crate) serial_service: Option<SerialService>,
    pub(crate) excel_service: Option<ExcelService>,
    sqlite_service: SqliteService,
    pub(crate) config_model: ConfigModel,
}

impl WorkService {
    pub fn new() -> Self {
        init_logger().unwrap();
        let config = WorkService::load_config().unwrap_or_else(|_| ConfigModel::default());
        let mut this = WorkService {
            serial_service: Option::None,
            sqlite_service: SqliteService::new("data.db".to_string()).unwrap(),
            excel_service: Option::None,
            config_model: config,
        };
        if this.config_model.excel_path != "" {
            this.init_excel_service(this.config_model.excel_path.clone()).unwrap();
        }
        return this;
    }
    pub fn get_port_names() -> Vec<String> {
        //maybe should not use unwrap
        return SerialService::get_ports().unwrap();
    }

    pub fn init_excel_service(&mut self, path: String) -> Result<bool, String> {
        return if let Ok(excel_service) = ExcelService::new(&path) {
            self.excel_service = Some(excel_service);
            self.config_model.excel_path = path;
            Ok(true)
        } else {
            Err("Cant Init ExcelService".to_string())
        };
    }

    pub fn write_and_check(&mut self, model: WorkModel) -> Result<bool, String> {
        let msg = &format!("{}:{}:{}", model.imei, model.sn, model.line_number);
        log::info!("Write Start");
        match self.sqlite_service.check_imei_has_insert(&model.imei) {
            Ok(true) => {
                log::error!("Imei Exist: {}", model.imei);
                self.serial_service = None;
                return Err("Imei Exist".to_string());
            }
            _ => {log::info!("Check Imei From Db Pass")}
        }
        match self.sqlite_service.check_sn_has_insert(&model.sn) {
            Ok(true) => {
                log::error!("Imei Exist: {}", model.sn);
                self.serial_service = None;
                return Err("Imei Exist".to_string());
            }
            _ => {log::info!("Check Sn From Db Pass")}
        }
        if let Err(msg) = self.write_imei(model.imei.clone()) {
            log::error!("Write Imei Error:{}", msg);
            return Err(msg);
        }
        log::info!("Write Imei:{}", model.imei);
        if let Err(msg) = self.write_sn(model.sn.clone()) {
            log::error!("Write Sn Error:{}", msg);
            return Err(msg);
        }
        log::info!("Write Imei:{}", model.sn);
        log::info!("Write End");
        log::info!("Check Start");
        if let Err(msg) = self.check_imei(&model.imei) {
            log::error!("Check Imei Error:{}", msg);
            return Err(msg);
        }
        log::info!("Check Imei:{}", model.imei);
        if let Err(msg) = self.check_sn(&model.sn) {
            log::error!("Check Sn Error:{}", msg);
            return Err(msg);
        }
        log::info!("Check Sn:{}", model.sn);
        log::info!("Check End");
        self.serial_service = None;
        match self.sqlite_service.insert(model) {
            Err(msg) => { log::error!("Insert Error:{}",msg); }
            _ => {}
        }
        Ok(true)
    }

    pub fn get_current_line(&mut self) -> usize {
        if let Some(service) = &mut self.excel_service {
            return service.get_current_line();
        }
        return 0;
    }

    pub fn set_current_line(&mut self, line: usize) -> Result<(), String> {
        if let Some(service) = &mut self.excel_service {
            return service.set_current_line(line);
        } else {
            Err("Cant Set Line".to_string())
        }
    }

    pub fn get_next_work_model(&mut self) -> Option<WorkModel> {
        if let Some(service) = &mut self.excel_service {
            if let Ok(model) = service.get_next_work_model() {
                self.config_model.current_line = service.get_current_line();
                return Some(model);
            } else { return None; }
        }
        return None;
    }

    pub fn roll_back(&mut self) {
        return if let Some(service) = &mut self.excel_service {
            service.roll_back();
        };
    }

    pub fn init_sqlite_service(&mut self, db_path: String) -> Result<bool, String> {
        return if let Ok(sqlite_service) = SqliteService::new(db_path) {
            self.sqlite_service = sqlite_service;
            Ok(true)
        } else {
            Err("Cant Init SqliteService".to_string())
        };
    }

    pub fn init_serial_service(&mut self, port_name: String) -> Result<bool, String> {
        return if let Ok(serial_service) = SerialService::new(&port_name) {
            self.serial_service = Some(*serial_service);
            self.config_model.last_com = port_name;
            Ok(true)
        } else {
            Err("Cant Init SerialService".to_string())
        };
    }

    fn write_imei(&mut self, imei: String) -> Result<bool, String> {
        let write_imei_command = format!("AT+ECCGSN=\"IMEI\",\"{}\"", imei);
        if let Some(service) = &mut self.serial_service {
            return service.send_command_with_target(write_imei_command, None, "OK".to_string());
        }
        return Err("Cant Init SerialService".to_string());
    }
    fn write_sn(&mut self, sn: String) -> Result<bool, String> {
        let write_sn_command = format!("AT+ECCGSN=\"SN\",\"{}\"", sn);
        if let Some(service) = &mut self.serial_service {
            return service.send_command_with_target(write_sn_command, None, "OK".to_string());
        }
        return Err("Cant Init SerialService".to_string());
    }

    fn check_imei(&mut self, imei: &str) -> Result<bool, String> {
        let check_imei_command = "AT+CGSN=1".to_string();
        if let Some(service) = &mut self.serial_service {
            return service.send_command_with_target(check_imei_command, None, imei.to_string());
        }
        return Err("Cant Init SerialService".to_string());
    }

    fn check_sn(&mut self, sn: &str) -> Result<bool, String> {
        let check_sn_command = "AT+CGSN=0".to_string();
        if let Some(service) = &mut self.serial_service {
            return service.send_command_with_target(check_sn_command, None, sn.to_string());
        }
        return Err("Cant Init ServialService".to_string());
    }

    pub fn save_config(&self) {
        ConfigService::save_config(self.config_model.clone()).unwrap()
    }

    fn load_config() -> Result<ConfigModel> {
        return ConfigService::load_config();
    }
}